/*
  PWMBase.h - PWM Base class
  Caution: SIGUSR1 signal setup required.

  Copyright (c) 2020 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#ifndef __PWMBASE_H
#define __PWMBASE_H

#include <stdint.h>

#define PWM_ACTIVE_LOW  0
#define PWM_ACTIVE_HIGH 1

#define PWM_DEF_FREQUENCY 490

#define PWM_DUTY_0   0x00000
#define PWM_DUTY_50  0x08000
#define PWM_DUTY_100 0x10000

class PWMBase
{
  protected:

    uint16_t _pin;
    int      _polarity;
    uint32_t _frequency;
    uint32_t _duty;
    bool     _setup;

  public:

    PWMBase(uint16_t pin)
    : _pin(pin)
    , _polarity(PWM_ACTIVE_HIGH)
    , _frequency(PWM_DEF_FREQUENCY)
    , _duty(PWM_DUTY_50)
    , _setup(false)
    {
    }

    virtual ~PWMBase(void)
    {
    }

    virtual void begin(int mode = 0)
    {
    }

    virtual void end(void)
    {
    }

    virtual void polarity(int value)
    {
      if (_polarity != value)
      {
        _polarity = value;
        _setup = true;
      }
    }

    virtual void frequency(uint32_t value)
    {
      if (_frequency != value)
      {
        _frequency = value;
        _setup = true;
      }
    }

    virtual void duty(uint32_t value)
    {
      if (value > PWM_DUTY_100)
        value = PWM_DUTY_100;
      if (_duty != value)
      {
        _duty = value;
        _setup = true;
      }
    }
};

#endif
