/*
  ThreadInterrupt.h - Software Interrupt functions
  Caution: SIGUSR1 signal setup required.

  Copyright (c) 2020 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#ifndef __THREADINTERRUPT_H
#define __THREADINTERRUPT_H

#include "ThreadBase.h"
#include "sysfs_gpio.h"

typedef void (*THREADINTERRUPT_FUNC_T)(void);

class ThreadInterrupt : public ThreadBase
{
  private:

    uint16_t _pin;
    THREADINTERRUPT_FUNC_T _func;
    SYSFS_GPIO_EDGE _mode;

    void *run(void)
    {
      set_thread_priority(SCHED_RR, sched_get_priority_max(SCHED_RR));
      sysfs_gpio::begin(_pin);
      if (sysfs_gpio::edge(_pin, _mode) == 0)
      {
        while (sysfs_gpio::poll(_pin, _func, -1) != -1)
          continue;
        sysfs_gpio::edge(_pin, SYSFS_GPIO_EDGE_NONE);
      }
//      sysfs_gpio::end(_pin);
      return NULL;
    }

  public:

    ThreadInterrupt(uint16_t pin, THREADINTERRUPT_FUNC_T func, SYSFS_GPIO_EDGE mode)
    : _pin(pin)
    , _func(func)
    , _mode(mode)
    {
    }

    virtual ~ThreadInterrupt(void)
    {
    }
};

#endif
