/*
  ThreadPWM.h - Software PWM functions
  Caution: SIGUSR1 signal setup required.

  Copyright (c) 2020 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#ifndef __THREADPWM_H
#define __THREADPWM_H

#include <stdint.h>
#include <time.h>
#include "Arduino.h"
#include "PWMBase.h"
#include "ThreadBase.h"
#include "Allwinner.h"

#define THREADPWM_MIN_FRQUENCY 1
#define THREADPWM_MAX_FRQUENCY 1000

class ThreadPWM : public ThreadBase, public PWMBase
{
  private:

    struct timespec _next;

    int sleep(unsigned long us)
    {
      _next.tv_sec  += (us / 1000000);
      _next.tv_nsec += (us % 1000000) * 1000;
      if (_next.tv_nsec >= 1000000000)
      {
        _next.tv_nsec -= 1000000000;
        _next.tv_sec++;
      }
      return clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &_next, NULL);
    }

    void *run(void)
    {
      uint32_t w, h, l;
      uint8_t  v;
      set_thread_priority(SCHED_RR, sched_get_priority_max(SCHED_RR));
      GPIO.func(_pin, GPIO_FUNC_OUTPUT);
      clock_gettime(CLOCK_MONOTONIC, &_next);
      _setup = true;
      while (1)
      {
        if (_setup)
        {
          _setup = false;
          w = 1000000 / _frequency;
          h = (uint64_t)w * _duty / PWM_DUTY_100;
          l = w - h;
          v = _polarity;
        }
        if (h)
        {
          GPIO.write(_pin, v);
          if (sleep(h))
            break;
        }
        if (l)
        {
          GPIO.write(_pin, !v);
          if (sleep(l))
            break;
        }
      }
      return NULL;
    }

  public:

    ThreadPWM(uint16_t pin) : PWMBase(pin)
    {
    }

    virtual ~ThreadPWM(void)
    {
    }

    void begin(int mode = 0)
    {
      ThreadBase::begin();
    }

    void end(void)
    {
      ThreadBase::end();
    }

    void frequency(uint32_t value)
    {
      if (value < THREADPWM_MIN_FRQUENCY)
        value = THREADPWM_MIN_FRQUENCY;
      else if (value > THREADPWM_MAX_FRQUENCY)
        value = THREADPWM_MAX_FRQUENCY;
      PWMBase::frequency(value);
    }
};

#endif
