/*
  ThreadTimer.h - Software Timer functions
  Caution: SIGUSR1 signal setup required.

  Copyright (c) 2020 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#ifndef __THREADTIMER_H
#define __THREADTIMER_H

#include <time.h>
#include "ThreadBase.h"

typedef unsigned long (*THREADTIMER_FUNC_T)(void *args, unsigned long us);

class ThreadTimer : public ThreadBase
{
  private:

    struct timespec _next;
    unsigned long _micros;
    THREADTIMER_FUNC_T _func;
    void *_args;

    void *run(void)
    {
      set_thread_priority(SCHED_RR, sched_get_priority_max(SCHED_RR));
      while (1)
      {
        _next.tv_sec  += (_micros / 1000000);
        _next.tv_nsec += (_micros % 1000000) * 1000;
        if (_next.tv_nsec >= 1000000000)
        {
          _next.tv_nsec -= 1000000000;
          _next.tv_sec++;
        }
        if (clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &_next, NULL))
          break;
        _micros = _func(_args, _micros);
        if (_micros == 0)
          break;
      }
      return NULL;
    }

  public:

    ThreadTimer(void)
    : _micros(0)
    , _func(NULL)
    , _args(NULL)
    {
    }

    virtual ~ThreadTimer(void)
    {
    }

    void begin(unsigned long us, THREADTIMER_FUNC_T func, void *args = NULL)
    {
      if (!running())
      {
        clock_gettime(CLOCK_MONOTONIC, &_next);
        _micros = us;
        _func   = func;
        _args   = args;
        ThreadBase::begin();
      }
    }
};

#endif
