/*
  Part of the Wiring project - http://wiring.uniandes.edu.co
  Caution: SIGUSR1 signal setup required.

  Copyright (c) 2020 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <signal.h>
#include <pthread.h>
#include "Arduino.h"
#include "ThreadInterrupt.h"

struct intr_info
{
  uint16_t pin;
  ThreadInterrupt *isr;
  struct intr_info *next;
};

static struct intr_info *intr_list = NULL;

static void add_intr(struct intr_info *intr)
{
  intr->next = intr_list;
  intr_list  = intr;
}

static struct intr_info *remove_intr(uint16_t pin)
{
  for (struct intr_info **p = &intr_list; *p; p = &(*p)->next)
  {
    if ((*p)->pin == pin)
    {
      struct intr_info *intr = *p;
      *p = (*p)->next;
      return intr;
    }
  }
  return NULL;
}

extern "C" void detachInterrupt(uint16_t pin)
{
  struct intr_info *intr = remove_intr(pin);
  if (intr)
  {
    intr->isr->end();
    delete intr->isr;
    free(intr);
  }
}

extern "C" void attachInterrupt(uint16_t pin, void (*func)(void), int mode)
{
  detachInterrupt(pin);
  if (func)
  {
    SYSFS_GPIO_EDGE edge;
    switch (mode)
    {
      case CHANGE:
        edge = SYSFS_GPIO_EDGE_BOTH;
        break;
      case FALLING:
        edge = SYSFS_GPIO_EDGE_FALLING;
        break;
      case RISING:
        edge = SYSFS_GPIO_EDGE_RISING;
        break;
      default:
        return;
    }
    struct intr_info *intr = (struct intr_info *)malloc(sizeof(*intr));
    if (intr)
    {
      intr->pin = pin;
      intr->isr = new ThreadInterrupt(pin, func, edge);
      intr->isr->begin();
      add_intr(intr);
    }
  }
}

extern "C" void interrupts(void)
{
}

extern "C" void noInterrupts(void)
{
}

