/*
  main.cpp - Main loop for Arduino sketches

  Copyright (c) 2020 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <Arduino.h>
#include <sched.h>
#include <signal.h>

int  __argc;
char **__argv;

static int _signum;

void init(void) __attribute__((weak));
void init(void)
{
}

void initVariant(void) __attribute__((weak));
void initVariant(void)
{
}

static void sig_handler(int signum)
{
  switch (signum)
  {
    case SIGPIPE:
    case SIGUSR1:
      return;
  }
  _signum = signum;
}

static void sig_init(void)
{
  struct sigaction act;
  memset(&act, 0, sizeof(act));
  act.sa_handler = sig_handler;
  act.sa_flags   = SA_RESTART;
  sigaction(SIGINT , &act, NULL);
  sigaction(SIGQUIT, &act, NULL);
  sigaction(SIGTERM, &act, NULL);
  sigaction(SIGPIPE, &act, NULL); // peer socket closed.
  sigaction(SIGUSR1, &act, NULL); // thread terminating signal.
}

int interrupted(void)
{
  if (serialEventRun)
    serialEventRun();
  return _signum;
}

void yield(void)
{
  interrupted();
  sched_yield();
}

int main(int argc, char **argv)
{
  __argc = argc;
  __argv = argv;
  sig_init();
  init();
  initVariant();
  setup();
  while (!interrupted())
    loop();
  return 0;
}

