/*
  wiring.c - Partial implementation of the Wiring API for the Linux
  Part of Arduino - http://www.arduino.cc/

  Copyright (c) 2020 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include <time.h>
#include <errno.h>

unsigned long micros(void)
{
  struct timespec ts;

  clock_gettime(CLOCK_MONOTONIC, &ts);
  return (unsigned long)((unsigned long long)ts.tv_sec * 1000000) + (ts.tv_nsec / 1000);
}

unsigned long millis(void)
{
  struct timespec ts;

  clock_gettime(CLOCK_MONOTONIC, &ts);
  return (unsigned long)((unsigned long long)ts.tv_sec * 1000) + (ts.tv_nsec / 1000000);
}

void delayMicroseconds(unsigned int us)
{
  struct timespec ts;

  clock_gettime(CLOCK_MONOTONIC, &ts);
  ts.tv_sec  += (us / 1000000);
  ts.tv_nsec += (us % 1000000) * 1000;
  if (ts.tv_nsec >= 1000000000)
  {
    ts.tv_nsec -= 1000000000;
    ts.tv_sec++;
  }
  clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &ts, NULL);
}

void delay(unsigned long ms)
{
  struct timespec ts;

  clock_gettime(CLOCK_MONOTONIC, &ts);
  ts.tv_sec  += (ms / 1000);
  ts.tv_nsec += (ms % 1000) * 1000000;
  if (ts.tv_nsec >= 1000000000)
  {
    ts.tv_nsec -= 1000000000;
    ts.tv_sec++;
  }
  clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &ts, NULL);
}

