/*
  wiring_digital.cpp - digital input and output functions
  Part of Arduino - http://www.arduino.cc/

  Copyright (c) 2020 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "Arduino.h"
#include "PWMCTRL.h"
#include "Allwinner.h"

extern "C" void pinMode(uint16_t pin, uint8_t mode)
{
  PWM.end(pin);
  switch (mode)
  {
    case OUTPUT:
      GPIO.func(pin, GPIO_FUNC_OUTPUT);
      break;
    case INPUT:
      GPIO.pull(pin, GPIO_PULL_DISABLE);
      GPIO.func(pin, GPIO_FUNC_INPUT);
      break;
    case INPUT_PULLUP:
      GPIO.pull(pin, GPIO_PULL_UP);
      GPIO.func(pin, GPIO_FUNC_INPUT);
      break;
    case INPUT_PULLDOWN:
      GPIO.pull(pin, GPIO_PULL_DOWN);
      GPIO.func(pin, GPIO_FUNC_INPUT);
      break;
  }
}

extern "C" void digitalWrite(uint16_t pin, uint8_t val)
{
  PWM.end(pin);
  GPIO.write(pin, val);
}

extern "C" int digitalRead(uint16_t pin)
{
  PWM.end(pin);
  return GPIO.read(pin);
}
